@@ -22,6 +22,7 @@ from objectid import ObjectId
 from cursor import Cursor
 from son import SON
 from errors import InvalidName, OperationFailure
+from code import Code
 
 _ZERO = "\x00\x00\x00\x00"
 _ONE = "\x01\x00\x00\x00"
@@ -320,7 +321,7 @@ class Collection(object):
         """
         return u"_".join([u"%s_%s" % item for item in keys])
 
-    def create_index(self, key_or_list, direction=None):
+    def create_index(self, key_or_list, direction=None, unique=False):
         """Creates an index on this collection.
 
         Takes either a single key and a direction, or a list of (key, direction)
@@ -333,12 +334,14 @@ class Collection(object):
             specifying the index to ensure
           - `direction` (optional): must be included if key_or_list is a single
             key, otherwise must be None
+          - `unique` (optional): should this index guarantee uniqueness?
         """
         to_save = SON()
         keys = pymongo._index_list(key_or_list, direction)
         to_save["name"] = self._gen_index_name(keys)
         to_save["ns"] = self.full_name()
         to_save["key"] = pymongo._index_document(keys)
+        to_save["unique"] = unique
 
         self.__database.system.indexes.save(to_save, False)
         return to_save["name"]
@@ -408,6 +411,44 @@ class Collection(object):
 
         return options
 
+    def group(self, keys, condition, initial, reduce):
+        """Perform a query similar to an SQL group by operation.
+
+        Returns an array of grouped items.
+
+        :Parameters:
+          - `keys`: list of fields to group by
+          - `condition`: specification of rows to be considered (as a `find`
+            query specification)
+          - `initial`: initial value of the aggregation counter object
+          - `reduce`: aggregation function as a JavaScript string
+        """
+        group_function = """function () {
+    var c = db[ns].find(condition);
+    var map = new Map();
+    var reduce_function = %s;
+    while (c.hasNext()) {
+        var obj = c.next();
+
+        var key = {};
+        for (var i in keys) {
+            key[keys[i]] = obj[keys[i]];
+        }
+
+        var aggObj = map[key];
+        if (aggObj == null) {
+            var newObj = Object.extend({}, key);
+            aggObj = map[key] = Object.extend(newObj, initial);
+        }
+        reduce_function(obj, aggObj);
+    }
+    return {"result": map.values()};
+}""" % reduce
+        return self.__database.eval(Code(group_function, {"ns": self.__collection_name,
+                                                          "keys": keys,
+                                                          "condition": condition,
+                                                          "initial": initial}))["result"]
+
     def __iter__(self):
         return self
 
